@@ -9,6 +9,8 @@ Module to communicate with the AVM Fritz!Box.
 
 import os
 import string
+import xml.etree.ElementTree as ElementTree
+
 import requests
 from requests.auth import HTTPDigestAuth
 
@@ -30,10 +32,18 @@ urllib3.disable_warnings()
 FRITZ_IP_ADDRESS = "169.254.1.1"
 FRITZ_TCP_PORT = 49000
 FRITZ_TLS_PORT = 49443
-FRITZ_USERNAME = "dslf-config"
+FRITZ_USERNAME = "dslf-config"  # for Fritz!OS < 7.24
 FRITZ_IGD_DESC_FILE = "igddesc.xml"
 FRITZ_TR64_DESC_FILE = "tr64desc.xml"
 FRITZ_DESCRIPTIONS = [FRITZ_IGD_DESC_FILE, FRITZ_TR64_DESC_FILE]
+FRITZ_USERNAME_REQUIRED_VERSION = 7.24
+
+# same defaults as used by requests:
+DEFAULT_POOL_CONNECTIONS = 10
+DEFAULT_POOL_MAXSIZE = 10
+
+# supported protocols:
+PROTOCOLS = ['http://', 'https://']
 
 
 class FritzConnection:
@@ -55,6 +65,18 @@ class FritzConnection:
     `use_tls` accepts a boolean for using encrypted communication with
     the Fritz!Box. Default is `False`.
     (`New in version 1.2`)
+
+    For some actions the Fritz!Box needs a password and since Fritz!OS
+    7.24 also requires a username, the previous default username is just
+    valid for OS versions < 7.24. In case the username is not given and
+    the system version is 7.24 or newer, FritzConnection uses the last
+    logged in username as default.
+    (`New in version 1.5`)
+
+    For applications where the urllib3 default connection-pool size
+    should get adapted, the arguments `pool_connections` and
+    `pool_maxsize` can get set explicitly.
+    (`New in version 1.6`)
     """
 
     def __init__(
@@ -65,6 +87,8 @@ class FritzConnection:
         password=None,
         timeout=None,
         use_tls=False,
+        pool_connections=DEFAULT_POOL_CONNECTIONS,
+        pool_maxsize=DEFAULT_POOL_MAXSIZE,
     ):
         """
         Initialisation of FritzConnection: reads all data from the box
@@ -89,7 +113,9 @@ class FritzConnection:
         communication with the router. In case of a timeout a
         `requests.ConnectTimeout` exception gets raised. `use_tls`
         accepts a boolean for using encrypted communication with the
-        Fritz!Box. Default is `False`.
+        Fritz!Box. Default is `False`. `pool_connections` and `pool_maxsize`
+        accept integers for changing the default urllib3 settings in order
+        to modify the number of reusable connections.
         """
         if address is None:
             address = FRITZ_IP_ADDRESS
@@ -103,12 +129,15 @@ class FritzConnection:
             port = FRITZ_TCP_PORT
         address = self.set_protocol(address, use_tls)
 
-        # session is optional but will speed up connections
-        # (significantly for tls):
+        # a session will speed up connections (significantly for tls)
+        # and is required to change the default poolsize:
         session = requests.Session()
         session.verify = False
         if password:
             session.auth = HTTPDigestAuth(user, password)
+        adapter = requests.adapters.HTTPAdapter(
+            pool_connections=pool_connections, pool_maxsize=pool_maxsize)
+        session.mount(PROTOCOLS[use_tls], adapter)
         # store as instance attributes for use by library modules
         self.address = address
         self.session = session
@@ -133,6 +162,9 @@ class FritzConnection:
 
         self.device_manager.scan()
         self.device_manager.load_service_descriptions(address, port)
+        # set default user for FritzOS >= 7.24:
+        self._reset_user(user, password)
+
 
     def __repr__(self):
         """Return a readable representation"""
@@ -182,17 +214,42 @@ class FritzConnection:
         and returns the modified `url`. Does not check whether the `url`
         given as parameter is correct.
         """
-        http = "http://"
-        https = "https://"
-        if url.startswith(http):
-            url = url[len(http) :]
-        elif url.startswith(https):
-            url = url[len(https) :]
-        if use_tls:
-            url = f"{https}{url}"
-        else:
-            url = f"{http}{url}"
-        return url
+        url = url.split("//", 1)[-1]
+        return PROTOCOLS[use_tls] + url
+
+    def _reset_user(self, user, password):
+        """
+        For Fritz!OS >= 7.24: if a password is given and the username is
+        the historic FRITZ_USERNAME, then check for the last logged-in
+        username and use this username for the soaper. Also recreate the
+        session used by the soaper and the device_manager.
+
+        This may not guarantee a valid user/password combination, but is
+        the way AVM recommends setting the required username in case a
+        username is not provided.
+        """
+        try:
+            sys_version = float(self.system_version)
+        except (ValueError, TypeError):
+            # version not available: don't do anything
+            return
+        if (sys_version >= FRITZ_USERNAME_REQUIRED_VERSION
+            and user == FRITZ_USERNAME
+            and password
+        ):
+            last_user = None
+            response = self.call_action('LANConfigSecurity1', 'X_AVM-DE_GetUserList')
+            root = ElementTree.fromstring(response['NewX_AVM-DE_UserList'])
+            for node in root:
+                if node.tag == 'Username' and node.attrib['last_user'] == '1':
+                    last_user = node.text
+                    break
+            if last_user is not None:
+                self.session.auth = HTTPDigestAuth(last_user, password)
+                self.soaper.user = last_user
+                self.soaper.session = self.session
+                self.device_manager.session = self.session
+
 
     # -------------------------------------------
     # public api:
